# swagger-java-client

Weather Source History API
- API version: 2.0.0
  - Build date: 2020-04-29T18:32:46.781Z[GMT]

Daily and hourly historical weather data for latitude/longitude points, Zip/Postal Codes, Designated Market Areas, and OnPoint™ points. The Weather Source History API is built upon the [OnPoint™ Platform](https://weathersource.com/products/onpoint-platform/) which ensures data that is gap-free, homogeneous, and ready for immediate analysis. We offer the highest resolution grid on the market, covering every landmass in the world and up to 200 miles offshore.  An API key is required to authorize requests. If you do not have a Weather Source API key [sign up for a free 30-day developer account](https://developer.weathersourceapis.com/account/sign-up/). 

  For more information, please visit [https://developer.weathersourceapis.com/](https://developer.weathersourceapis.com/)

*Automatically generated by the [Swagger Codegen](https://github.com/swagger-api/swagger-codegen)*


## Requirements

Building the API client library requires:
1. Java 1.7+
2. Maven/Gradle

## Installation

To install the API client library to your local Maven repository, simply execute:

```shell
mvn clean install
```

To deploy it to a remote Maven repository instead, configure the settings of the repository and execute:

```shell
mvn clean deploy
```

Refer to the [OSSRH Guide](http://central.sonatype.org/pages/ossrh-guide.html) for more information.

### Maven users

Add this dependency to your project's POM:

```xml
<dependency>
  <groupId>io.swagger</groupId>
  <artifactId>swagger-java-client</artifactId>
  <version>1.0.0</version>
  <scope>compile</scope>
</dependency>
```

### Gradle users

Add this dependency to your project's build file:

```groovy
compile "io.swagger:swagger-java-client:1.0.0"
```

### Others

At first generate the JAR by executing:

```shell
mvn clean package
```

Then manually install the following JARs:

* `target/swagger-java-client-1.0.0.jar`
* `target/lib/*.jar`

## Getting Started

Please follow the [installation](#installation) instruction and execute the following Java code:

```java
import io.swagger.client.*;
import io.swagger.client.auth.*;
import io.swagger.client.model.*;
import io.swagger.client.api.DmasApi;

import java.io.File;
import java.util.*;

public class DmasApiExample {

    public static void main(String[] args) {
        ApiClient defaultClient = Configuration.getDefaultApiClient();

        // Configure API key authorization: ApiKeyAuth
        ApiKeyAuth ApiKeyAuth = (ApiKeyAuth) defaultClient.getAuthentication("ApiKeyAuth");
        ApiKeyAuth.setApiKey("YOUR API KEY");
        // Uncomment the following line to set a prefix for the API key, e.g. "Token" (defaults to null)
        //ApiKeyAuth.setApiKeyPrefix("Token");

        DmasApi apiInstance = new DmasApi();
        Integer dmaId = 56; // Integer | A [Designated Market Area ID](https://developer.weathersourceapis.com/apis/supported-dmas/).
        LocalDate date = new LocalDate(); // LocalDate | An date formatted as a RFC3339 date value.
        List<String> fields = Arrays.asList("fields_example"); // List<String> | A comma separated list of field names to return. The date field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are supported: * all *(all fields)* * allCldCvr *(derivatives of cldCvr)* * allHum *(derivatives of relHum, spcHum)* * allPrecip *(precip, snowDepth, snowfall)* * allPres *(derivatives of mslPres, presTend, sfcPres)* * allRad *(derivatives of radSolar)* * allTemp *(derivatives of dewPt, feelsLike, heatIndex, temp, wetBulb, windChill)* * allWind *(derivatives of windDir, windDir80m, windDir100m, windSpd, windSpd80m, windSpd100m)* * popular *(derivatives of cldCvr, feelsLike, precip, relHum, snowfall, temp, windDir, windSpd)* 
        String unitScale = "IMPERIAL"; // String | The unit scale for returned values.<br>
        try {
            HistoryDmaDayObj result = apiInstance.dmasDmaIdDaysDateGet(dmaId, date, fields, unitScale);
            System.out.println(result);
        } catch (ApiException e) {
            System.err.println("Exception when calling DmasApi#dmasDmaIdDaysDateGet");
            e.printStackTrace();
        }
    }
}
import io.swagger.client.*;
import io.swagger.client.auth.*;
import io.swagger.client.model.*;
import io.swagger.client.api.DmasApi;

import java.io.File;
import java.util.*;

public class DmasApiExample {

    public static void main(String[] args) {
        ApiClient defaultClient = Configuration.getDefaultApiClient();

        // Configure API key authorization: ApiKeyAuth
        ApiKeyAuth ApiKeyAuth = (ApiKeyAuth) defaultClient.getAuthentication("ApiKeyAuth");
        ApiKeyAuth.setApiKey("YOUR API KEY");
        // Uncomment the following line to set a prefix for the API key, e.g. "Token" (defaults to null)
        //ApiKeyAuth.setApiKeyPrefix("Token");

        DmasApi apiInstance = new DmasApi();
        Integer dmaId = 56; // Integer | A [Designated Market Area ID](https://developer.weathersourceapis.com/apis/supported-dmas/).
        LocalDate dateStart = new LocalDate(); // LocalDate | An date formatted as a RFC3339 date value that must occur temporally before dateEnd.
        LocalDate dateEnd = new LocalDate(); // LocalDate | An date formatted as a RFC3339 date value that must occur temporally after dateStart.
        List<String> fields = Arrays.asList("fields_example"); // List<String> | A comma separated list of field names to return. The date field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are supported: * all *(all fields)* * allCldCvr *(derivatives of cldCvr)* * allHum *(derivatives of relHum, spcHum)* * allPrecip *(precip, snowDepth, snowfall)* * allPres *(derivatives of mslPres, presTend, sfcPres)* * allRad *(derivatives of radSolar)* * allTemp *(derivatives of dewPt, feelsLike, heatIndex, temp, wetBulb, windChill)* * allWind *(derivatives of windDir, windDir80m, windDir100m, windSpd, windSpd80m, windSpd100m)* * popular *(derivatives of cldCvr, feelsLike, precip, relHum, snowfall, temp, windDir, windSpd)* 
        String unitScale = "IMPERIAL"; // String | The unit scale for returned values.<br>
        try {
            HistoryDmaDayObj result = apiInstance.dmasDmaIdDaysDateStartdateEndGet(dmaId, dateStart, dateEnd, fields, unitScale);
            System.out.println(result);
        } catch (ApiException e) {
            System.err.println("Exception when calling DmasApi#dmasDmaIdDaysDateStartdateEndGet");
            e.printStackTrace();
        }
    }
}
import io.swagger.client.*;
import io.swagger.client.auth.*;
import io.swagger.client.model.*;
import io.swagger.client.api.DmasApi;

import java.io.File;
import java.util.*;

public class DmasApiExample {

    public static void main(String[] args) {
        ApiClient defaultClient = Configuration.getDefaultApiClient();

        // Configure API key authorization: ApiKeyAuth
        ApiKeyAuth ApiKeyAuth = (ApiKeyAuth) defaultClient.getAuthentication("ApiKeyAuth");
        ApiKeyAuth.setApiKey("YOUR API KEY");
        // Uncomment the following line to set a prefix for the API key, e.g. "Token" (defaults to null)
        //ApiKeyAuth.setApiKeyPrefix("Token");

        DmasApi apiInstance = new DmasApi();
        Integer dmaId = 56; // Integer | A [Designated Market Area ID](https://developer.weathersourceapis.com/apis/supported-dmas/).
        OffsetDateTime timestamp = new OffsetDateTime(); // OffsetDateTime | A timestamp formatted as an RFC3339 date-time value. The UTC offset is required.
        List<String> fields = Arrays.asList("fields_example"); // List<String> | A comma separated list of field names to return. The timestamp field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are also supported: * all *(all fields)* * allCldCvr *(cldCvr)* * allHum *(relHum, spcHum)* * allPrecip *(precip, snowfall)* * allPres *(mslPres, presTend, sfcPres)* * allRad *(radSolar)* * allTemp *(dewPt, feelsLike, heatIndex, temp, wetBulb, windChill)* * allWind *(windDir, windDir80m, windDir100m, windSpd, windSpd80m, windSpd100m)* * popular *(cldCvr, feelsLike, precip, relHum, snowfall, temp, windDir, windSpd)* 
        String unitScale = "IMPERIAL"; // String | The unit scale for returned values.<br>
        try {
            HistoryDmaHourObj result = apiInstance.dmasDmaIdHoursTimestampGet(dmaId, timestamp, fields, unitScale);
            System.out.println(result);
        } catch (ApiException e) {
            System.err.println("Exception when calling DmasApi#dmasDmaIdHoursTimestampGet");
            e.printStackTrace();
        }
    }
}
import io.swagger.client.*;
import io.swagger.client.auth.*;
import io.swagger.client.model.*;
import io.swagger.client.api.DmasApi;

import java.io.File;
import java.util.*;

public class DmasApiExample {

    public static void main(String[] args) {
        ApiClient defaultClient = Configuration.getDefaultApiClient();

        // Configure API key authorization: ApiKeyAuth
        ApiKeyAuth ApiKeyAuth = (ApiKeyAuth) defaultClient.getAuthentication("ApiKeyAuth");
        ApiKeyAuth.setApiKey("YOUR API KEY");
        // Uncomment the following line to set a prefix for the API key, e.g. "Token" (defaults to null)
        //ApiKeyAuth.setApiKeyPrefix("Token");

        DmasApi apiInstance = new DmasApi();
        Integer dmaId = 56; // Integer | A [Designated Market Area ID](https://developer.weathersourceapis.com/apis/supported-dmas/).
        OffsetDateTime timestampStart = new OffsetDateTime(); // OffsetDateTime | A timestamp formatted as an RFC3339 date-time value that must occur temporally before timestampEnd. The UTC offset is required.
        OffsetDateTime timestampEnd = new OffsetDateTime(); // OffsetDateTime | A timestamp formatted as an RFC3339 date-time value that must occur temporally after timestampStart. The UTC offset is required.
        List<String> fields = Arrays.asList("fields_example"); // List<String> | A comma separated list of field names to return. The timestamp field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are also supported: * all *(all fields)* * allCldCvr *(cldCvr)* * allHum *(relHum, spcHum)* * allPrecip *(precip, snowfall)* * allPres *(mslPres, presTend, sfcPres)* * allRad *(radSolar)* * allTemp *(dewPt, feelsLike, heatIndex, temp, wetBulb, windChill)* * allWind *(windDir, windDir80m, windDir100m, windSpd, windSpd80m, windSpd100m)* * popular *(cldCvr, feelsLike, precip, relHum, snowfall, temp, windDir, windSpd)* 
        String unitScale = "IMPERIAL"; // String | The unit scale for returned values.<br>
        try {
            HistoryDmaHourObj result = apiInstance.dmasDmaIdHoursTimestampStarttimestampEndGet(dmaId, timestampStart, timestampEnd, fields, unitScale);
            System.out.println(result);
        } catch (ApiException e) {
            System.err.println("Exception when calling DmasApi#dmasDmaIdHoursTimestampStarttimestampEndGet");
            e.printStackTrace();
        }
    }
}
```

## Documentation for API Endpoints

All URIs are relative to *https://history.weathersourceapis.com/v2*

Class | Method | HTTP request | Description
------------ | ------------- | ------------- | -------------
*DmasApi* | [**dmasDmaIdDaysDateGet**](docs/DmasApi.md#dmasDmaIdDaysDateGet) | **GET** /dmas/{dmaId}/days/{date} | A day of weather history for a DMA.
*DmasApi* | [**dmasDmaIdDaysDateStartdateEndGet**](docs/DmasApi.md#dmasDmaIdDaysDateStartdateEndGet) | **GET** /dmas/{dmaId}/days/{dateStart},{dateEnd} | A contiguous range of daily weather history for a DMA.
*DmasApi* | [**dmasDmaIdHoursTimestampGet**](docs/DmasApi.md#dmasDmaIdHoursTimestampGet) | **GET** /dmas/{dmaId}/hours/{timestamp} | An hour of weather history for a DMA
*DmasApi* | [**dmasDmaIdHoursTimestampStarttimestampEndGet**](docs/DmasApi.md#dmasDmaIdHoursTimestampStarttimestampEndGet) | **GET** /dmas/{dmaId}/hours/{timestampStart},{timestampEnd} | A contiguous range of hourly weather history for a DMA.
*OnpointsApi* | [**onpointsOnpointIdDaysDateGet**](docs/OnpointsApi.md#onpointsOnpointIdDaysDateGet) | **GET** /onpoints/{onpointId}/days/{date} | A day of weather history for an OnPoint™ point.
*OnpointsApi* | [**onpointsOnpointIdDaysDateStartdateEndGet**](docs/OnpointsApi.md#onpointsOnpointIdDaysDateStartdateEndGet) | **GET** /onpoints/{onpointId}/days/{dateStart},{dateEnd} | A contiguous range of daily weather history for an OnPoint™ point.
*OnpointsApi* | [**onpointsOnpointIdHoursTimestampGet**](docs/OnpointsApi.md#onpointsOnpointIdHoursTimestampGet) | **GET** /onpoints/{onpointId}/hours/{timestamp} | An hour of weather history for an OnPoint™ point.
*OnpointsApi* | [**onpointsOnpointIdHoursTimestampStarttimestampEndGet**](docs/OnpointsApi.md#onpointsOnpointIdHoursTimestampStarttimestampEndGet) | **GET** /onpoints/{onpointId}/hours/{timestampStart},{timestampEnd} | A contiguous range of hourly weather history for an OnPoint™ point.
*PointsApi* | [**pointsLatitudelongitudeDaysDateGet**](docs/PointsApi.md#pointsLatitudelongitudeDaysDateGet) | **GET** /points/{latitude},{longitude}/days/{date} | A day of weather history for a Latitude/Longitude point.
*PointsApi* | [**pointsLatitudelongitudeDaysDateStartdateEndGet**](docs/PointsApi.md#pointsLatitudelongitudeDaysDateStartdateEndGet) | **GET** /points/{latitude},{longitude}/days/{dateStart},{dateEnd} | A contiguous range of daily weather history for a Latitude/Longitude point.
*PointsApi* | [**pointsLatitudelongitudeHoursTimestampGet**](docs/PointsApi.md#pointsLatitudelongitudeHoursTimestampGet) | **GET** /points/{latitude},{longitude}/hours/{timestamp} | An hour of weather history for a Latitude/Longitude point.
*PointsApi* | [**pointsLatitudelongitudeHoursTimestampStarttimestampEndGet**](docs/PointsApi.md#pointsLatitudelongitudeHoursTimestampStarttimestampEndGet) | **GET** /points/{latitude},{longitude}/hours/{timestampStart},{timestampEnd} | A contiguous range of hourly weather history for a Latitude/Longitude point.
*PostcodesApi* | [**postcodesPostcodecountryCodeDaysDateGet**](docs/PostcodesApi.md#postcodesPostcodecountryCodeDaysDateGet) | **GET** /postcodes/{postcode},{countryCode}/days/{date} | A day of weather history for a postcode.
*PostcodesApi* | [**postcodesPostcodecountryCodeDaysDateStartdateEndGet**](docs/PostcodesApi.md#postcodesPostcodecountryCodeDaysDateStartdateEndGet) | **GET** /postcodes/{postcode},{countryCode}/days/{dateStart},{dateEnd} | A contiguous range of daily weather history for a postcode.
*PostcodesApi* | [**postcodesPostcodecountryCodeHoursTimestampGet**](docs/PostcodesApi.md#postcodesPostcodecountryCodeHoursTimestampGet) | **GET** /postcodes/{postcode},{countryCode}/hours/{timestamp} | An hour of weather history for a postcode.
*PostcodesApi* | [**postcodesPostcodecountryCodeHoursTimestampStarttimestampEndGet**](docs/PostcodesApi.md#postcodesPostcodecountryCodeHoursTimestampStarttimestampEndGet) | **GET** /postcodes/{postcode},{countryCode}/hours/{timestampStart},{timestampEnd} | A contiguous range of hourly weather history for a postcode.

## Documentation for Models

 - [BoundingPoints](docs/BoundingPoints.md)
 - [BoundingPointsInner](docs/BoundingPointsInner.md)
 - [DateRangeObj](docs/DateRangeObj.md)
 - [DmaObj](docs/DmaObj.md)
 - [ErrorObj](docs/ErrorObj.md)
 - [FieldListDayObj](docs/FieldListDayObj.md)
 - [FieldListHourObj](docs/FieldListHourObj.md)
 - [Fields](docs/Fields.md)
 - [Grid](docs/Grid.md)
 - [HistoryDayArr](docs/HistoryDayArr.md)
 - [HistoryDayObj](docs/HistoryDayObj.md)
 - [HistoryDmaDayObj](docs/HistoryDmaDayObj.md)
 - [HistoryDmaHourObj](docs/HistoryDmaHourObj.md)
 - [HistoryHourArr](docs/HistoryHourArr.md)
 - [HistoryHourObj](docs/HistoryHourObj.md)
 - [HistoryOnpointDayObj](docs/HistoryOnpointDayObj.md)
 - [HistoryOnpointHourObj](docs/HistoryOnpointHourObj.md)
 - [HistoryPointDayObj](docs/HistoryPointDayObj.md)
 - [HistoryPointHourObj](docs/HistoryPointHourObj.md)
 - [HistoryPostcodeDayObj](docs/HistoryPostcodeDayObj.md)
 - [HistoryPostcodeHourObj](docs/HistoryPostcodeHourObj.md)
 - [OnpointObj](docs/OnpointObj.md)
 - [PointObj](docs/PointObj.md)
 - [PostcodeObj](docs/PostcodeObj.md)
 - [SamplePoints](docs/SamplePoints.md)
 - [SamplePointsInner](docs/SamplePointsInner.md)
 - [TimestampRangeObj](docs/TimestampRangeObj.md)

## Documentation for Authorization

Authentication schemes defined for the API:
### ApiKeyAuth

- **Type**: API key
- **API key parameter name**: X-API-KEY
- **Location**: HTTP header


## Recommendation

It's recommended to create an instance of `ApiClient` per thread in a multithreaded environment to avoid any potential issues.

## Author

support@weathersource.com
